// weyl_remap.cc -- remap a matrix for the computation of its Weyl symbol

// To use this file, your version of Octave must support dynamic
// linking.  To find out if it does, type the command
//
//   x = octave_config_info; x.DEFS
//
// at the Octave prompt.  Support for dynamic linking is included if
// the output contains the string -DWITH_DYNAMIC_LINKING=1.
//
// To compile this file, type the command
//
// mkoctfile --verbose weyl_remap.cc
//
// at the shell prompt.  The script mkoctfile should have been
// installed along with Octave.  Running it will create a file called
// weyl_remap.oct that can be loaded by Octave.
//
// Enter:
//
// help weyl_remap
//
// at the Octave prompt to have a description of how weyl_remap should be used.  
// 
// This file may be used under the terms defined by the GNU 
// General Public License [write to the Free Software Foundation, 
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA]
//
// WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//
// Author: Eric Chassande-Mottin [ecm@obs-nice.fr], Mon Feb 23 17:23:08 GMT 2004


#include <octave/config.h>

#include <iostream>
#include <sstream>
using namespace std;
#include <string>

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>

static inline int
getmin ( int x, int y)
{
  return x < y ? x : y;
}

DEFUN_DLD (weyl_remap, args, ,
  "WEYL_REMAP: remap a matrix for the computation of its Weyl symbol.\n")
{
  octave_value_list retval;
  
  /*----------- get input parameters ----------------*/
  int nargin = args.length ();
  
  if (nargin < 1)
    {
      error("Usage: out = weyl_remap(in)");
      return retval;
    }

  if (args(0).is_complex_type())
    {
      /* get the data */ 
      ComplexMatrix in(args(0).complex_matrix_value());
  
      /* create output data */
      int c=in.columns();
      int r=in.rows();
      int N=0;
      if (c==r)
	N=c;
      else
	{
	  error("weyl_remap: matrix must be square");
	  return retval;
	}
      int N2=2*N;
      
      ComplexMatrix out(N2,N,0);
      
      /* remap matrix */
      int kmax;
      for (int n=0 ; n<N ; n++)
	{    
	  kmax=getmin(2*n,N2-1-2*n);
	  for (int k=-kmax; k<=kmax ; k++)
	    out((N2+k)%N2,n)=in((int) floor(n+.5*(double) k),(int) floor(n-.5*(double)k));
	}
  
      retval(0)=out;
    }
  else
    {
      /* get the data */
      Matrix in(args(0).matrix_value());
      
      /* create output data */
      int c=in.columns();
      int r=in.rows();
      int N=0;
      if (c==r)
	N=c;
      else
	{
	  error("weyl_remap: matrix must be square");
	  return retval;
	}
      int N2=2*N;
      
      Matrix out(N2,N,0);
      
      /* remap matrix */
      int kmax;
      for (int n=0 ; n<N ; n++)
	{    
	  kmax=getmin(2*n,N2-1-2*n);
	  for (int k=-kmax; k<=kmax ; k++)
	    out((N2+k)%N2,n)=in((int) floor(n+.5*(double) k),(int) floor(n-.5*(double)k));
	}
      
      retval(0)=out;
    }

  return retval;
}
